@@ -13,11 +13,13 @@ from trytond.tests.test_tryton import doctest_checker
 from trytond.transaction import Transaction
 from trytond.exceptions import UserError
 
-from trytond.modules.company.tests import create_company, set_company
+from trytond.modules.account.tax import TaxableMixin
+from trytond.modules.company.tests import (
+    create_company, set_company, PartyCompanyCheckEraseMixin, CompanyTestMixin)
 from trytond.modules.currency.tests import create_currency
 
 
-def create_chart(company, tax=False):
+def create_chart(company, tax=False, chart='account.account_template_root_en'):
     pool = Pool()
     AccountTemplate = pool.get('account.account.template')
     TaxTemplate = pool.get('account.tax.template')
@@ -26,20 +28,12 @@ def create_chart(company, tax=False):
     CreateChart = pool.get('account.create_chart', type='wizard')
     Account = pool.get('account.account')
 
-    template = AccountTemplate(ModelData.get_id(
-            'account', 'account_template_root_en'))
+    module, xml_id = chart.split('.')
+    template = AccountTemplate(ModelData.get_id(module, xml_id))
     if tax:
         tax_account = AccountTemplate(ModelData.get_id(
                 'account', 'account_template_tax_en'))
         with Transaction().set_user(0):
-            tax_code = TaxCodeTemplate()
-            tax_code.name = 'Tax Code'
-            tax_code.account = template
-            tax_code.save()
-            base_code = TaxCodeTemplate()
-            base_code.name = 'Base Code'
-            base_code.account = template
-            base_code.save()
             tax = TaxTemplate()
             tax.name = tax.description = '20% VAT'
             tax.type = 'percentage'
@@ -47,51 +41,73 @@ def create_chart(company, tax=False):
             tax.account = template
             tax.invoice_account = tax_account
             tax.credit_note_account = tax_account
-            tax.invoice_base_code = base_code
-            tax.invoice_base_sign = Decimal(1)
-            tax.invoice_tax_code = tax_code
-            tax.invoice_tax_sign = Decimal(1)
-            tax.credit_note_base_code = base_code
-            tax.credit_note_base_sign = Decimal(-1)
-            tax.credit_note_tax_code = tax_code
-            tax.credit_note_tax_sign = Decimal(-1)
             tax.save()
 
+            TaxCodeTemplate.create([{
+                        'name': 'Tax Code',
+                        'account': template,
+                        'lines': [('create', [{
+                                        'operator': '+',
+                                        'type': 'invoice',
+                                        'amount': 'tax',
+                                        'tax': tax.id,
+                                        }])],
+                        }, {
+                        'name': 'Base Code',
+                        'account': template,
+                        'lines': [('create', [{
+                                        'operator': '+',
+                                        'type': 'invoice',
+                                        'amount': 'base',
+                                        'tax': tax.id,
+                                        }])],
+                        }])
+
     session_id, _, _ = CreateChart.create()
     create_chart = CreateChart(session_id)
     create_chart.account.account_template = template
     create_chart.account.company = company
     create_chart.transition_create_account()
     receivable, = Account.search([
-            ('kind', '=', 'receivable'),
+            ('type.receivable', '=', True),
+            ('party_required', '=', True),
             ('company', '=', company.id),
-            ])
+            ], limit=1)
     payable, = Account.search([
-            ('kind', '=', 'payable'),
+            ('type.payable', '=', True),
+            ('party_required', '=', True),
             ('company', '=', company.id),
-            ])
+            ], limit=1)
     create_chart.properties.company = company
     create_chart.properties.account_receivable = receivable
     create_chart.properties.account_payable = payable
     create_chart.transition_create_properties()
 
 
-def get_fiscalyear(company, today=None):
+def get_fiscalyear(company, today=None, start_date=None, end_date=None):
     pool = Pool()
     Sequence = pool.get('ir.sequence')
+    SequenceType = pool.get('ir.sequence.type')
     FiscalYear = pool.get('account.fiscalyear')
 
     if not today:
         today = datetime.date.today()
+    if not start_date:
+        start_date = today.replace(month=1, day=1)
+    if not end_date:
+        end_date = today.replace(month=12, day=31)
 
+    sequence_type, = SequenceType.search([
+            ('name', '=', "Account Move"),
+            ], limit=1)
     sequence, = Sequence.create([{
                 'name': '%s' % today.year,
-                'code': 'account.move',
+                'sequence_type': sequence_type.id,
                 'company': company.id,
                 }])
     fiscalyear = FiscalYear(name='%s' % today.year, company=company)
-    fiscalyear.start_date = today.replace(month=1, day=1)
-    fiscalyear.end_date = today.replace(month=12, day=31)
+    fiscalyear.start_date = start_date
+    fiscalyear.end_date = end_date
     fiscalyear.post_move_sequence = sequence
     return fiscalyear
 
@@ -110,8 +126,8 @@ def close_fiscalyear(fiscalyear):
 
     # Balance non-deferral
     journal_sequence, = Sequence.search([
-            ('code', '=', 'account.journal'),
-            ])
+            ('sequence_type.name', '=', "Account Journal"),
+            ], limit=1)
     journal_closing, = Journal.create([{
                 'name': 'Closing',
                 'code': 'CLO',
@@ -129,14 +145,12 @@ def close_fiscalyear(fiscalyear):
             ('name', '=', 'Equity'),
             ])
     revenue, = Account.search([
-            ('kind', '=', 'revenue'),
+            ('type.revenue', '=', True),
             ])
     account_pl, = Account.create([{
                 'name': 'P&L',
                 'type': type_equity.id,
-                'deferral': True,
                 'parent': revenue.parent.id,
-                'kind': 'other',
                 }])
 
     session_id = BalanceNonDeferral.create()[0]
@@ -160,7 +174,8 @@ def close_fiscalyear(fiscalyear):
     FiscalYear.close([fiscalyear])
 
 
-class AccountTestCase(ModuleTestCase):
+class AccountTestCase(
+        PartyCompanyCheckEraseMixin, CompanyTestMixin, ModuleTestCase):
     'Test Account module'
     module = 'account'
 
@@ -202,6 +217,45 @@ class AccountTestCase(ModuleTestCase):
             FiscalYear.create_period([fiscalyear])
             self.assertEqual(len(fiscalyear.periods), 12)
 
+    @with_transaction()
+    def test_fiscalyear_create_periods(self):
+        'Test fiscalyear create periods'
+        FiscalYear = Pool().get('account.fiscalyear')
+
+        company = create_company()
+        with set_company(company):
+            year = datetime.date.today().year
+            date = datetime.date
+            for start_date, end_date, interval, end_day, num_periods in [
+                    (date(year, 1, 1), date(year, 12, 31), 1, 31, 12),
+                    (date(year + 1, 1, 1), date(year + 1, 12, 31), 3, 31, 4),
+                    (date(year + 2, 1, 1), date(year + 2, 12, 31), 5, 31, 3),
+                    (date(year + 3, 4, 6), date(year + 4, 4, 5), 1, 5, 12),
+                    (date(year + 4, 4, 6), date(year + 5, 4, 5), 3, 5, 4),
+                    (date(year + 5, 4, 6), date(year + 6, 4, 5), 5, 5, 3),
+                    (date(year + 6, 6, 6), date(year + 6, 12, 31), 1, 29, 8),
+                    (date(year + 7, 7, 7), date(year + 7, 12, 31), 3, 29, 3),
+                    (date(year + 8, 1, 1), date(year + 9, 8, 7), 1, 29, 20),
+                    (date(year + 9, 9, 9), date(year + 10, 11, 12), 3, 29, 5),
+                    ]:
+                fiscalyear = get_fiscalyear(
+                    company, start_date, start_date, end_date)
+                fiscalyear.save()
+                FiscalYear.create_period([fiscalyear], interval, end_day)
+
+                self.assertEqual(len(fiscalyear.periods), num_periods)
+
+                self.assertEqual(fiscalyear.periods[-1].end_date, end_date)
+                self.assertTrue(all(
+                    p.end_date == p.end_date + relativedelta(day=end_day)
+                    for p in fiscalyear.periods[:-1]))
+
+                self.assertEqual(fiscalyear.periods[0].start_date, start_date)
+                self.assertTrue(all(
+                    p1.end_date + relativedelta(days=1) == p2.start_date
+                    for p1, p2 in zip(
+                        fiscalyear.periods[:-1], fiscalyear.periods[1:])))
+
     @with_transaction()
     def test_account_debit_credit(self):
         'Test account debit/credit'
@@ -232,19 +286,18 @@ class AccountTestCase(ModuleTestCase):
                     ('code', '=', 'EXP'),
                     ])
             revenue, = Account.search([
-                    ('kind', '=', 'revenue'),
+                    ('type.revenue', '=', True),
                     ])
             receivable, = Account.search([
-                    ('kind', '=', 'receivable'),
+                    ('type.receivable', '=', True),
                     ])
             expense, = Account.search([
-                    ('kind', '=', 'expense'),
+                    ('type.expense', '=', True),
                     ])
             payable, = Account.search([
-                    ('kind', '=', 'payable'),
+                    ('type.payable', '=', True),
                     ])
             cash, = Account.search([
-                    ('kind', '=', 'other'),
                     ('name', '=', 'Main Cash'),
                     ])
             cash_cur, = Account.copy([cash], default={
@@ -409,6 +462,78 @@ class AccountTestCase(ModuleTestCase):
                 self.assertEqual(
                     cash_cur.amount_second_currency, Decimal(50))
 
+    @with_transaction()
+    def test_account_type_amount(self):
+        "Test account type amount"
+        pool = Pool()
+        Party = pool.get('party.party')
+        FiscalYear = pool.get('account.fiscalyear')
+        Journal = pool.get('account.journal')
+        Account = pool.get('account.account')
+        Move = pool.get('account.move')
+        Type = pool.get('account.account.type')
+
+        party = Party(name='Party')
+        party.save()
+
+        company = create_company()
+        with set_company(company):
+            fiscalyear = get_fiscalyear(company)
+            fiscalyear.save()
+            FiscalYear.create_period([fiscalyear])
+            period = fiscalyear.periods[0]
+            create_chart(company)
+
+            journal_revenue, = Journal.search([
+                    ('code', '=', 'REV'),
+                    ])
+            revenue, = Account.search([
+                    ('type.revenue', '=', True),
+                    ])
+            receivable, = Account.search([
+                    ('type.receivable', '=', True),
+                    ])
+
+            Move.create([{
+                        'period': period.id,
+                        'journal': journal_revenue.id,
+                        'date': period.start_date,
+                        'lines': [
+                            ('create', [{
+                                        'account': revenue.id,
+                                        'credit': Decimal(100),
+                                        }, {
+                                        'account': receivable.id,
+                                        'debit': Decimal(100),
+                                        'party': party.id,
+                                        }]),
+                            ],
+                        }])
+
+            with Transaction().set_context(fiscalyear=fiscalyear.id):
+                revenue_type = Type(revenue.type)
+                receivable_type = Type(receivable.type)
+
+            # Test type amount
+            self.assertEqual(revenue_type.amount, Decimal(100))
+            self.assertEqual(receivable_type.amount, Decimal(100))
+
+            # Set a debit type on receivable
+            with Transaction().set_context(fiscalyear=fiscalyear.id):
+                debit_receivable_type, = Type.copy([receivable_type])
+            receivable.debit_type = debit_receivable_type
+            receivable.save()
+            self.assertEqual(receivable_type.amount, Decimal(0))
+            self.assertEqual(debit_receivable_type.amount, Decimal(100))
+
+            # Set a debit type on revenue
+            with Transaction().set_context(fiscalyear=fiscalyear.id):
+                debit_revenue_type, = Type.copy([revenue_type])
+            revenue.debit_type = debit_revenue_type
+            revenue.save()
+            self.assertEqual(revenue_type.amount, Decimal(100))
+            self.assertEqual(debit_revenue_type.amount, Decimal(0))
+
     @with_transaction()
     def test_move_post(self):
         "Test posting move"
@@ -436,10 +561,10 @@ class AccountTestCase(ModuleTestCase):
                     ('code', '=', 'REV'),
                     ])
             revenue, = Account.search([
-                    ('kind', '=', 'revenue'),
+                    ('type.revenue', '=', True),
                     ])
             receivable, = Account.search([
-                    ('kind', '=', 'receivable'),
+                    ('type.receivable', '=', True),
                     ])
 
             move = Move()
@@ -882,6 +1007,131 @@ class AccountTestCase(ModuleTestCase):
                     [vat0, ecotax1, vat1, ecotax3, vat2]),
                 Decimal(100))
 
+    class Taxable(TaxableMixin):
+        __slots__ = ('currency', 'taxable_lines')
+
+        def __init__(self, currency=None, taxable_lines=None):
+            super().__init__()
+            self.currency = currency
+            self.taxable_lines = taxable_lines
+
+    @with_transaction()
+    def test_taxable_mixin_line(self):
+        "Test TaxableMixin with rounding on line"
+        pool = Pool()
+        Tax = pool.get('account.tax')
+        Configuration = pool.get('account.configuration')
+        currency = create_currency('cur')
+
+        company = create_company()
+        with set_company(company):
+            create_chart(company, tax=True)
+            tax, = Tax.search([])
+            config = Configuration(1)
+            config.tax_rounding = 'line'
+            config.save()
+
+            taxable = self.Taxable(
+                currency=currency,
+                taxable_lines=[
+                    ([tax], Decimal('1.001'), 1, None),
+                    ] * 100)
+
+            taxes = taxable._get_taxes()
+
+        tax, = taxes
+        self.assertEqual(tax['base'], Decimal('100.00'))
+        self.assertEqual(tax['amount'], Decimal('20.00'))
+
+    @with_transaction()
+    def test_taxable_mixin_document(self):
+        "Test TaxableMixin with rounding on document"
+        pool = Pool()
+        Tax = pool.get('account.tax')
+        Configuration = pool.get('account.configuration')
+        currency = create_currency('cur')
+
+        class Taxable(TaxableMixin):
+            __slots__ = ('currency', 'taxable_lines')
+
+            def __init__(self, currency=None, taxable_lines=None):
+                super().__init__()
+                self.currency = currency
+                self.taxable_lines = taxable_lines
+
+        company = create_company()
+        with set_company(company):
+            create_chart(company, tax=True)
+            tax, = Tax.search([])
+            config = Configuration(1)
+            config.tax_rounding = 'document'
+            config.save()
+
+            taxable = Taxable(
+                currency=currency,
+                taxable_lines=[
+                    ([tax], Decimal('1.001'), 1, None),
+                    ] * 100)
+
+            taxes = taxable._get_taxes()
+
+        tax, = taxes
+        self.assertEqual(tax['base'], Decimal('100.00'))
+        self.assertEqual(tax['amount'], Decimal('20.02'))
+
+    @with_transaction()
+    def test_tax_compute_with_children_update_unit_price(self):
+        "Test tax compute with children taxes modifying unit_price"
+        pool = Pool()
+        Account = pool.get('account.account')
+        Tax = pool.get('account.tax')
+
+        company = create_company()
+        with set_company(company):
+            create_chart(company)
+
+            tax_account, = Account.search([
+                    ('name', '=', 'Main Tax'),
+                    ])
+
+            tax1 = Tax()
+            tax1.name = tax1.description = "Tax 1"
+            tax1.type = 'none'
+            tax1.update_unit_price = True
+            tax1.sequence = 1
+            tax1.save()
+            child1 = Tax()
+            child1.name = child1.description = "Child 1"
+            child1.type = 'percentage'
+            child1.rate = Decimal('0.1')
+            child1.invoice_account = tax_account
+            child1.credit_note_account = tax_account
+            child1.parent = tax1
+            child1.save()
+
+            tax2 = Tax()
+            tax2.name = tax2.description = "Tax 2"
+            tax2.type = 'fixed'
+            tax2.amount = Decimal('10')
+            tax2.invoice_account = tax_account
+            tax2.credit_note_account = tax_account
+            tax2.sequence = 2
+            tax2.save()
+
+            self.assertEqual(
+                Tax.compute([tax1, tax2], Decimal(100), 2), [{
+                        'base': Decimal(200),
+                        'amount': Decimal(20),
+                        'tax': child1,
+                        }, {
+                        'base': Decimal('220'),
+                        'amount': Decimal('20'),
+                        'tax': tax2,
+                        }])
+            self.assertEqual(
+                Tax.reverse_compute(Decimal('120'), [tax1, tax2]),
+                Decimal('100'))
+
     @with_transaction()
     def test_receivable_payable(self):
         'Test party receivable payable'
@@ -906,16 +1156,16 @@ class AccountTestCase(ModuleTestCase):
                     ('code', '=', 'EXP'),
                     ])
             revenue, = Account.search([
-                    ('kind', '=', 'revenue'),
+                    ('type.revenue', '=', True),
                     ])
             receivable, = Account.search([
-                    ('kind', '=', 'receivable'),
+                    ('type.receivable', '=', True),
                     ])
             expense, = Account.search([
-                    ('kind', '=', 'expense'),
+                    ('type.expense', '=', True),
                     ])
             payable, = Account.search([
-                    ('kind', '=', 'payable'),
+                    ('type.payable', '=', True),
                     ])
             party, = Party.create([{
                         'name': 'Receivable/Payable party',
@@ -1010,10 +1260,10 @@ class AccountTestCase(ModuleTestCase):
         with set_company(company):
             create_chart(company)
             receivable, = Account.search([
-                    ('kind', '=', 'receivable'),
+                    ('type.receivable', '=', True),
                     ])
             payable, = Account.search([
-                    ('kind', '=', 'payable'),
+                    ('type.payable', '=', True),
                     ])
 
             party = Party(party.id)
@@ -1044,6 +1294,70 @@ class AccountTestCase(ModuleTestCase):
                         }])
             self.assertListEqual(tax_rule.apply(tax, {}), [target_tax.id])
 
+    @with_transaction()
+    def test_tax_rule_start_date(self):
+        "Test tax rule start date"
+        pool = Pool()
+        TaxRule = pool.get('account.tax.rule')
+        Tax = pool.get('account.tax')
+        Date = pool.get('ir.date')
+
+        today = Date.today()
+        yesterday = today - datetime.timedelta(days=1)
+        tomorrow = today + datetime.timedelta(days=1)
+        company = create_company()
+        with set_company(company):
+            create_chart(company, tax=True)
+            tax, = Tax.search([])
+            target_tax, = Tax.copy([tax])
+
+            tax_rule, = TaxRule.create([{
+                        'name': "Test",
+                        'kind': 'both',
+                        'lines': [('create', [{
+                                        'start_date': today,
+                                        'tax': target_tax.id,
+                                        }])],
+                        }])
+
+            self.assertListEqual(tax_rule.apply(tax, {}), [target_tax.id])
+            self.assertListEqual(
+                tax_rule.apply(tax, {'date': yesterday}), [tax.id])
+            self.assertListEqual(
+                tax_rule.apply(tax, {'date': tomorrow}), [target_tax.id])
+
+    @with_transaction()
+    def test_tax_rule_end_date(self):
+        "Test tax rule end date"
+        pool = Pool()
+        TaxRule = pool.get('account.tax.rule')
+        Tax = pool.get('account.tax')
+        Date = pool.get('ir.date')
+
+        today = Date.today()
+        yesterday = today - datetime.timedelta(days=1)
+        tomorrow = today + datetime.timedelta(days=1)
+        company = create_company()
+        with set_company(company):
+            create_chart(company, tax=True)
+            tax, = Tax.search([])
+            target_tax, = Tax.copy([tax])
+
+            tax_rule, = TaxRule.create([{
+                        'name': "Test",
+                        'kind': 'both',
+                        'lines': [('create', [{
+                                        'end_date': today,
+                                        'tax': target_tax.id,
+                                        }])],
+                        }])
+
+            self.assertListEqual(tax_rule.apply(tax, {}), [target_tax.id])
+            self.assertListEqual(
+                tax_rule.apply(tax, {'date': yesterday}), [target_tax.id])
+            self.assertListEqual(
+                tax_rule.apply(tax, {'date': tomorrow}), [tax.id])
+
     @with_transaction()
     def test_tax_rule_keep_origin(self):
         "Test tax rule keeps origin"
@@ -1067,7 +1381,266 @@ class AccountTestCase(ModuleTestCase):
                                         }])],
                         }])
             self.assertListEqual(
-                tax_rule.apply(tax, {}),  [target_tax.id, tax.id])
+                tax_rule.apply(tax, {}), [target_tax.id, tax.id])
+
+    @with_transaction()
+    def test_update_chart(self):
+        'Test all template models are updated when updating chart'
+        pool = Pool()
+        TypeTemplate = pool.get('account.account.type.template')
+        AccountTemplate = pool.get('account.account.template')
+        TaxTemplate = pool.get('account.tax.template')
+        TaxCodeTemplate = pool.get('account.tax.code.template')
+        ModelData = pool.get('ir.model.data')
+        UpdateChart = pool.get('account.update_chart', type='wizard')
+        Type = pool.get('account.account.type')
+        Account = pool.get('account.account')
+        Tax = pool.get('account.tax')
+        TaxCode = pool.get('account.tax.code')
+
+        def check():
+            for type_ in Type.search([]):
+                self.assertEqual(type_.name, type_.template.name)
+                self.assertEqual(
+                    type_.statement, type_.template.statement)
+
+            for account in Account.search([]):
+                self.assertEqual(account.name, account.template.name)
+                self.assertEqual(account.code, account.template.code)
+                self.assertEqual(account.type.template, account.template.type)
+                self.assertEqual(account.reconcile, account.template.reconcile)
+                self.assertEqual(
+                    account.start_date, account.template.start_date)
+                self.assertEqual(account.end_date, account.template.end_date)
+                self.assertEqual(
+                    account.party_required, account.template.party_required)
+                self.assertEqual(
+                    account.general_ledger_balance,
+                    account.template.general_ledger_balance)
+                self.assertEqual(
+                    set(t.template for t in account.taxes),
+                    set(t for t in account.template.taxes))
+
+            for tax_code in TaxCode.search([]):
+                self.assertEqual(tax_code.name, tax_code.template.name)
+                self.assertEqual(tax_code.code, tax_code.template.code)
+                for line in tax_code.lines:
+                    self.assertEqual(line.code.template, line.template.code)
+                    self.assertEqual(line.operator, line.template.operator)
+                    self.assertEqual(line.type, line.template.type)
+                    self.assertEqual(line.tax.template, line.template.tax)
+
+            for tax in Tax.search([]):
+                self.assertEqual(tax.name, tax.template.name)
+                self.assertEqual(tax.description, tax.template.description)
+                self.assertEqual(tax.type, tax.template.type)
+                self.assertEqual(tax.rate, tax.template.rate)
+                self.assertEqual(tax.amount, tax.template.amount)
+                self.assertEqual(
+                    tax.update_unit_price, tax.template.update_unit_price)
+                self.assertEqual(
+                    tax.start_date, tax.template.start_date)
+                self.assertEqual(tax.end_date, tax.template.end_date)
+                self.assertEqual(
+                    tax.invoice_account.template, tax.template.invoice_account)
+                self.assertEqual(
+                    tax.credit_note_account.template,
+                    tax.template.credit_note_account)
+
+        company = create_company()
+        with set_company(company):
+            create_chart(company, True)
+
+            with Transaction().set_context(active_test=False):
+                self.assertEqual(Type.search([], count=True), 16)
+                self.assertEqual(Account.search([], count=True), 7)
+                self.assertEqual(Tax.search([], count=True), 1)
+
+                check()
+
+        with Transaction().set_user(0):
+            root_type = TypeTemplate(ModelData.get_id(
+                'account', 'account_type_template_minimal_en'))
+            root_type.name = 'Updated Minimal Chart'
+            root_type.save()
+            chart = AccountTemplate(ModelData.get_id(
+                    'account', 'account_template_root_en'))
+            new_type = TypeTemplate()
+            new_type.name = 'New Type'
+            new_type.parent = root_type
+            new_type.statement = 'balance'
+            new_type.save()
+            new_account = AccountTemplate()
+            new_account.name = 'New Account'
+            new_account.parent = chart
+            new_account.type = new_type
+            new_account.save()
+            updated_tax, = TaxTemplate.search([])
+            updated_tax.name = 'VAT'
+            updated_tax.invoice_account = new_account
+            updated_tax.save()
+            updated_account = AccountTemplate(ModelData.get_id(
+                    'account', 'account_template_revenue_en'))
+            updated_account.code = 'REV'
+            updated_account.name = 'Updated Account'
+            updated_account.reconcile = True
+            updated_account.end_date = datetime.date.today()
+            updated_account.taxes = [updated_tax]
+            updated_account.save()
+            inactive_account = AccountTemplate(ModelData.get_id(
+                    'account', 'account_template_expense_en'))
+            inactive_account.end_date = datetime.date.min
+            inactive_account.save()
+            new_tax = TaxTemplate()
+            new_tax.name = new_tax.description = '10% VAT'
+            new_tax.type = 'percentage'
+            new_tax.rate = Decimal('0.1')
+            new_tax.account = chart
+            new_tax.invoice_account = new_account
+            new_tax.credit_note_account = new_account
+            new_tax.save()
+            updated_tax_code, = TaxCodeTemplate.search([
+                    ('name', '=', 'Tax Code'),
+                    ])
+            updated_tax_code.name = 'Updated Tax Code'
+            updated_tax_code.save()
+            updated_tax_code_line, = updated_tax_code.lines
+            updated_tax_code_line.operator = '-'
+            updated_tax_code_line.save()
+
+        with set_company(company):
+            account, = Account.search([('parent', '=', None)])
+            session_id, _, _ = UpdateChart.create()
+            update_chart = UpdateChart(session_id)
+            update_chart.start.account = account
+            update_chart.transition_update()
+
+            with Transaction().set_context(active_test=False):
+                self.assertEqual(Type.search([], count=True), 17)
+                self.assertEqual(Account.search([], count=True), 8)
+                self.assertEqual(Tax.search([], count=True), 2)
+
+                check()
+
+    @with_transaction()
+    def test_update_override(self):
+        "Test all models are not updated when template override is True"
+        pool = Pool()
+        ModelData = pool.get('ir.model.data')
+        TypeTemplate = pool.get('account.account.type.template')
+        AccountTemplate = pool.get('account.account.template')
+        TaxTemplate = pool.get('account.tax.template')
+        TaxCodeTemplate = pool.get('account.tax.code.template')
+        UpdateChart = pool.get('account.update_chart', type='wizard')
+        Type = pool.get('account.account.type')
+        Account = pool.get('account.account')
+        Tax = pool.get('account.tax')
+        TaxCode = pool.get('account.tax.code')
+        TaxCodeLine = pool.get('account.tax.code.line')
+
+        new_name = "Updated"
+
+        company = create_company()
+        with set_company(company):
+            create_chart(company, True)
+
+            type_count = Type.search([], count=True)
+            account_count = Account.search([], count=True)
+            tax_count = Tax.search([], count=True)
+            tax_code_count = TaxCode.search([], count=True)
+            tax_code_line_count = TaxCodeLine.search([], count=True)
+
+        with Transaction().set_user(0):
+            root = AccountTemplate(ModelData.get_id(
+                    'account', 'account_template_root_en'))
+
+            template_type, = TypeTemplate.search([
+                    ('parent', '!=', None),
+                    ('parent', 'child_of', [root.type.id]),
+                    ], limit=1)
+            template_type.name = new_name
+            template_type.save()
+            type_, = Type.search([('template', '=', template_type.id)])
+            type_.template_override = True
+            type_.save()
+
+            template_account, = AccountTemplate.search([
+                    ('parent', '!=', None),
+                    ('parent', 'child_of', [root.id]),
+                    ], limit=1)
+            template_account.name = new_name
+            template_account.save()
+            account, = Account.search([('template', '=', template_account.id)])
+            account.template_override = True
+            account.save()
+
+            template_tax, = TaxTemplate.search([])
+            template_tax.name = new_name
+            template_tax.save()
+            tax, = Tax.search([('template', '=', template_tax.id)])
+            tax.template_override = True
+            tax.save()
+
+            template_tax_code, = TaxCodeTemplate.search([], limit=1)
+            template_tax_code.name = new_name
+            template_tax_code.save()
+            tax_code, = TaxCode.search(
+                [('template', '=', template_tax_code.id)])
+            tax_code.template_override = True
+            tax_code.save()
+
+            template_tax_code_line, = TaxCodeLine.search([], limit=1)
+            tax_code_line, = TaxCodeLine.search(
+                [('template', '=', template_tax_code_line.id)])
+            tax_code_line.template_override = True
+            tax_code_line.save()
+
+        with set_company(company):
+            account, = Account.search([('parent', '=', None)])
+            session_id, _, _ = UpdateChart.create()
+            update_chart = UpdateChart(session_id)
+            update_chart.start.account = account
+            update_chart.transition_update()
+
+            self.assertEqual(Type.search([], count=True), type_count)
+            self.assertEqual(Account.search([], count=True), account_count)
+            self.assertEqual(Tax.search([], count=True), tax_count)
+            self.assertEqual(
+                TaxCode.search([], count=True), tax_code_count)
+            self.assertEqual(
+                TaxCodeLine.search([], count=True), tax_code_line_count)
+
+            for Model in [Type, Account, Tax, TaxCode]:
+                for record in Model.search([]):
+                    self.assertNotEqual(record.name, new_name)
+
+    @with_transaction()
+    def test_update_inactive(self):
+        "Test update chart of accounts with inactive account"
+        pool = Pool()
+        Account = pool.get('account.account')
+        UpdateChart = pool.get('account.update_chart', type='wizard')
+
+        company = create_company()
+        with set_company(company):
+            create_chart(company, tax=True)
+            root, = Account.search([('parent', '=', None)])
+
+            cash, = Account.search([('name', '=', 'Main Cash')])
+            cash.template_override = True
+            cash.end_date = datetime.date.min
+            cash.save()
+            self.assertFalse(cash.active)
+
+            session_id, _, _ = UpdateChart.create()
+            update_chart = UpdateChart(session_id)
+            update_chart.start.account = root
+            update_chart.transition_update()
+
+            with Transaction().set_context(active_test=False):
+                self.assertEqual(
+                    Account.search([('name', '=', 'Main Cash')], count=True),
+                    1)
 
 
 def suite():
@@ -1079,11 +1652,21 @@ def suite():
             tearDown=doctest_teardown, encoding='utf-8',
             checker=doctest_checker,
             optionflags=doctest.REPORT_ONLY_FIRST_FAILURE))
+    suite.addTests(doctest.DocFileSuite(
+            'scenario_account_reconcile.rst',
+            tearDown=doctest_teardown, encoding='utf-8',
+            checker=doctest_checker,
+            optionflags=doctest.REPORT_ONLY_FIRST_FAILURE))
     suite.addTests(doctest.DocFileSuite(
             'scenario_move_cancel.rst',
             tearDown=doctest_teardown, encoding='utf-8',
             checker=doctest_checker,
             optionflags=doctest.REPORT_ONLY_FIRST_FAILURE))
+    suite.addTests(doctest.DocFileSuite(
+            'scenario_move_line_group.rst',
+            tearDown=doctest_teardown, encoding='utf-8',
+            checker=doctest_checker,
+            optionflags=doctest.REPORT_ONLY_FIRST_FAILURE))
     suite.addTests(doctest.DocFileSuite(
             'scenario_move_template.rst',
             tearDown=doctest_teardown, encoding='utf-8',
@@ -1099,4 +1682,14 @@ def suite():
             tearDown=doctest_teardown, encoding='utf-8',
             checker=doctest_checker,
             optionflags=doctest.REPORT_ONLY_FIRST_FAILURE))
+    suite.addTests(doctest.DocFileSuite(
+            'scenario_account_active.rst',
+            tearDown=doctest_teardown, encoding='utf-8',
+            checker=doctest_checker,
+            optionflags=doctest.REPORT_ONLY_FIRST_FAILURE))
+    suite.addTests(doctest.DocFileSuite(
+            'scenario_tax_code.rst',
+            tearDown=doctest_teardown, encoding='utf-8',
+            checker=doctest_checker,
+            optionflags=doctest.REPORT_ONLY_FIRST_FAILURE))
     return suite
